﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using ProjectExcavator.Entities;

namespace ProjectExcavator.DrawingObjects
{
    public class DrawingExcavator
    {
        /// <summary>
        /// Класс-сущность
        /// </summary>
        public EntityExcavator? EntityExcavator { get; protected set; }
        /// <summary>
        /// Ширина окна
        /// </summary>
        private int _pictureWidth;
        /// <summary>
        /// Высота окна
        /// </summary>
        private int _pictureHeight;
        /// <summary>
        /// /// Левая координата прорисовки автомобиля
        /// </summary>
        protected int _startPosX;
        /// <summary>
        /// Верхняя кооридната прорисовки автомобиля
        /// </summary>
        protected int _startPosY;
        /// <summary>
        /// Ширина прорисовки автомобиля
        /// </summary>
        protected readonly int _exWidth = 138;
        /// <summary>
        /// Высота прорисовки автомобиля
        /// </summary>
        protected readonly int _exHeight = 80;
        /// <summary>
        /// Инициализация свойств
        /// </summary>
        /// <param name="speed">Скорость</param>
        /// <param name="weight">Вес</param>
        /// <param name="bodyColor">Цвет кузова</param>
        /// <param name="width">Ширина картинки</param>
        /// <param name="height">Высота картинки</param>
        /// <returns>true - объект создан, false - проверка не пройдена, нельзя создать объект в этих размерах</returns>
        public DrawingExcavator(int speed, double weight, Color bodyColor, int width, int height)
        {
            // TODO: Продумать проверки
            if (width > _exWidth || height > _exHeight)
            {
                _pictureWidth = width;
                _pictureHeight = height;
                EntityExcavator = new EntityExcavator(speed, weight, bodyColor);
            }
        }

        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="speed">Скорость</param>
        /// <param name="weight">Вес</param>
        /// <param name="bodyColor">Основной цвет</param>
        /// <param name="width">Ширина картинки</param>
        /// <param name="height">Высота картинки</param>
        /// <param name="exWidth">Ширина прорисовки автомобиля</param>
        /// <param name="exHeight">Высота прорисовки автомобиля</param>
        protected DrawingExcavator(int speed, double weight, Color bodyColor, int
        width, int height, int exWidth, int exHeight)
        {
            // TODO: Продумать проверки
            if (width > _exWidth || height > _exHeight)
            {
                _pictureWidth = width;
                _pictureHeight = height;
                _exWidth = exWidth;
                _exHeight = exHeight;
                EntityExcavator = new EntityExcavator(speed, weight, bodyColor);
            }
        }

        /// <summary>
        /// Установка позиции
        /// </summary>
        /// <param name="x">Координата X</param>
        /// <param name="y">Координата Y</param>
        public void SetPosition(int x, int y)
        {
            // TODO: Изменение x, y
            if (x < 0)
            {
                x = 0;
            }
            else if (x > _pictureWidth - _exWidth)
            {
                x = _pictureWidth - _exWidth;
            }

            if (y < 0)
            {
                y = 0;
            }
            else if (y > _pictureHeight - _exHeight)
            {
                y = _pictureHeight - _exHeight;
            }
            _startPosX = x;
            _startPosY = y;
        }
        /// <summary>
        /// Координата X объекта
        /// </summary>
        public int GetPosX => _startPosX;
        /// <summary>
        /// Координата Y объекта
        /// </summary>
        public int GetPosY => _startPosY;
        /// <summary>
        /// Ширина объекта
        /// </summary>
        public int GetWidth => _exWidth;
        /// <summary>
        /// Высота объекта
        /// </summary>
        public int GetHeight => _exHeight;

        /// <summary>
        /// Проверка, что объект может переместится по указанному направлению
        /// </summary>
        /// <param name="direction">Направление</param>
        /// <returns>true - можно переместится по указанному направлению</returns>
        public bool CanMove(DirectionType direction)
        {
            if (EntityExcavator == null)
            {
                return false;
            }
            return direction switch
            {
                //влево
                DirectionType.Left => _startPosX - EntityExcavator.Step > 0,
                //вверх
                DirectionType.Up => _startPosY - EntityExcavator.Step > 0,
                // вправо
                DirectionType.Right => _startPosX + _exWidth + EntityExcavator.Step <= _pictureWidth,
                //влево
                DirectionType.Down => _startPosY + _exHeight + EntityExcavator.Step <= _pictureHeight,
            };
        }
        /// <summary>
        /// Изменение направления перемещения
        /// </summary>
        /// <param name="direction">Направление</param>
        public void MoveTransport(DirectionType direction)
        {
            if (!CanMove(direction) || EntityExcavator == null)
            {
                return;
            }
            switch (direction)
            {
                //влево
                case DirectionType.Left:
                    _startPosX -= (int)EntityExcavator.Step;
                    break;
                //вверх
                case DirectionType.Up:
                    _startPosY -= (int)EntityExcavator.Step;
                    break;
                // вправо
                case DirectionType.Right:
                    _startPosX += (int)EntityExcavator.Step;
                    break;
                //вниз
                case DirectionType.Down:
                    _startPosY += (int)EntityExcavator.Step;
                    break;
            }
        }
        /// <summary>
        /// Прорисовка объекта
        /// </summary>
        /// <param name="g"></param>
        public virtual void DrawTransport(Graphics g)
        {
            if (EntityExcavator == null)
            {
                return;
            }

            //цвета
            Pen pen = new(Color.Black);
            Brush brBlue = new SolidBrush(Color.LightBlue);
            Brush brYellow = new SolidBrush(Color.Yellow);
            Brush brGray = new SolidBrush(Color.Gray);
            Brush brBlack = new SolidBrush(Color.Black);

            //отрисовка экскаватора без ковша
            g.DrawRectangle(pen, _startPosX + 50, _startPosY + 35, 75, 25);
            g.DrawRectangle(pen, _startPosX + 95, _startPosY + 10, 30, 25);
            g.DrawRectangle(pen, _startPosX + 60, _startPosY + 15, 10, 20);
            g.DrawRectangle(pen, _startPosX + 44, _startPosY + 65, 86, 20);
            g.DrawPie(pen, _startPosX + 34, _startPosY + 65, 20, 20, 90, 180);
            g.DrawPie(pen, _startPosX + 120, _startPosY + 65, 20, 20, 270, 180);
            g.DrawEllipse(pen, _startPosX + 40, _startPosY + 68, 15, 15);
            g.DrawEllipse(pen, _startPosX + 120, _startPosY + 68, 15, 15);
            g.DrawEllipse(pen, _startPosX + 60, _startPosY + 76, 8, 8);
            g.DrawEllipse(pen, _startPosX + 80, _startPosY + 76, 8, 8);
            g.DrawEllipse(pen, _startPosX + 100, _startPosY + 76, 8, 8);
            g.DrawEllipse(pen, _startPosX + 72, _startPosY + 68, 6, 6);
            g.DrawEllipse(pen, _startPosX + 92, _startPosY + 68, 6, 6);

            //кабина водителя
            g.FillRectangle(brBlue, _startPosX + 96, _startPosY + 11, 29, 24);

            // кузов
            g.FillRectangle(brYellow, _startPosX + 51, _startPosY + 36, 74, 24);

            // труба
            g.FillRectangle(brYellow, _startPosX + 61, _startPosY + 16, 9, 19);

            //гусеница
            g.FillPie(brGray, _startPosX + 34, _startPosY + 65, 20, 20, 90, 180);
            g.FillPie(brGray, _startPosX + 120, _startPosY + 65, 20, 20, 270, 180);
            g.FillRectangle(brGray, _startPosX + 44, _startPosY + 65, 86, 20);
            g.FillEllipse(brBlack, _startPosX + 40, _startPosY + 68, 15, 15);
            g.FillEllipse(brBlack, _startPosX + 120, _startPosY + 68, 15, 15);
            g.FillEllipse(brBlack, _startPosX + 60, _startPosY + 76, 8, 8);
            g.FillEllipse(brBlack, _startPosX + 80, _startPosY + 76, 8, 8);
            g.FillEllipse(brBlack, _startPosX + 100, _startPosY + 76, 8, 8);
            g.FillEllipse(brBlack, _startPosX + 72, _startPosY + 68, 6, 6);
            g.FillEllipse(brBlack, _startPosX + 92, _startPosY + 68, 6, 6);
        }
    }
}
