﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using ProjectExcavator.Entities;

namespace ProjectExcavator.DrawingObjects
{
    /// <summary>
    /// Класс, отвечающий за прорисовку и перемещение объекта-сущности
    /// </summary>
    public class DrawingExcavatorKovsh : DrawingExcavator
    {
        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="speed">Скорость</param>
        /// <param name="weight">Вес</param>
        /// <param name="bodyColor">Основной цвет</param>
        /// <param name="additionalColor">Дополнительный цвет</param>
        /// <param name="kovsh">Признак наличия ковша</param>
        /// <param name="katki">Признак наличия катков</param>
        /// <param name="width">Ширина картинки</param>
        /// <param name="height">Высота картинки</param>
        public DrawingExcavatorKovsh(int speed, double weight, Color bodyColor, Color
       additionalColor, bool kovsh, bool katki, int width, int height) : base(speed, weight, bodyColor, width, height, 138, 80)
        {
            if (EntityExcavator != null)
            {
                EntityExcavator = new EntityExcavatorKovsh(speed, weight, bodyColor,
                additionalColor, kovsh, katki);
            }
        }
        public override void DrawTransport(Graphics g)
        {
            if (EntityExcavator is not EntityExcavatorKovsh excavatorKovsh)
            {
                return;
            }
            //цвета
            Pen pen = new(Color.Black);
            Brush additionalBrush = new
            SolidBrush(excavatorKovsh.AdditionalColor);
            Brush brBlack = new SolidBrush(Color.Black);

            // ковш
            if (excavatorKovsh.Kovsh)
            {
                g.DrawLine(pen, _startPosX + 50, _startPosY + 35, _startPosX + 10, _startPosY + 10);
                g.DrawLine(pen, _startPosX + 58, _startPosY + 35, _startPosX + 12, _startPosY + 5);
                g.DrawEllipse(pen, _startPosX + 7, _startPosY + 4, 7, 7);
                g.DrawLine(pen, _startPosX + 10, _startPosY + 10, _startPosX + 10, _startPosY + 45);
                g.DrawLine(pen, _startPosX + 14, _startPosY + 5, _startPosX + 14, _startPosY + 45);
                g.DrawPie(pen, _startPosX, _startPosY + 44, 28, 30, 90, 180);
                g.DrawLine(pen, _startPosX + 14, _startPosY + 5, _startPosX + 14, _startPosY);
                g.DrawLine(pen, _startPosX + 14, _startPosY, _startPosX + 7, _startPosY + 10);
                g.DrawLine(pen, _startPosX + 14, _startPosY, _startPosX + 50, _startPosY + 12);
                g.DrawLine(pen, _startPosX + 14, _startPosY, _startPosX + 50, _startPosY + 16);
                g.DrawLine(pen, _startPosX + 50, _startPosY + 12, _startPosX + 50, _startPosY + 29);

                g.FillEllipse(additionalBrush, _startPosX + 7, _startPosY + 4, 7, 7);
                g.FillPie(brBlack, _startPosX, _startPosY + 44, 28, 30, 90, 180);
                Point point1 = new Point(_startPosX + 50, _startPosY + 35);
                Point point2 = new Point(_startPosX + 10, _startPosY + 10);
                Point point3 = new Point(_startPosX + 12, _startPosY + 5);
                Point point4 = new Point(_startPosX + 58, _startPosY + 35);
                Point[] truba_1 = { point1, point2, point3, point4, point1 };
                g.FillPolygon(additionalBrush, truba_1);

                Point point5 = new Point(_startPosX + 10, _startPosY + 10);
                Point point6 = new Point(_startPosX + 10, _startPosY + 45);
                Point point7 = new Point(_startPosX + 14, _startPosY + 45);
                Point point8 = new Point(_startPosX + 14, _startPosY + 5);
                Point[] truba_2 = { point5, point6, point7, point8, point5 };
                g.FillPolygon(additionalBrush, truba_2);

                Point point9 = new Point(_startPosX + 14, _startPosY + 5);
                Point point10 = new Point(_startPosX + 14, _startPosY);
                Point point11 = new Point(_startPosX + 7, _startPosY + 10);
                Point[] triangle = { point9, point10, point11, point9 };
                g.FillPolygon(additionalBrush, triangle);

                Point point12 = new Point(_startPosX + 14, _startPosY);
                Point point13 = new Point(_startPosX + 50, _startPosY + 12);
                Point point14 = new Point(_startPosX + 50, _startPosY + 16);
                Point point15 = new Point(_startPosX + 14, _startPosY);
                Point[] krepl = { point12, point13, point14, point15, point12 };
                g.FillPolygon(additionalBrush, krepl);
            }
            base.DrawTransport(g);

            // катки
            if (excavatorKovsh.Katki)
            {
                g.FillEllipse(additionalBrush, _startPosX + 40, _startPosY + 68, 15, 15);
                g.FillEllipse(additionalBrush, _startPosX + 120, _startPosY + 68, 15, 15);
                g.FillEllipse(additionalBrush, _startPosX + 60, _startPosY + 76, 8, 8);
                g.FillEllipse(additionalBrush, _startPosX + 80, _startPosY + 76, 8, 8);
                g.FillEllipse(additionalBrush, _startPosX + 100, _startPosY + 76, 8, 8);
                g.FillEllipse(additionalBrush, _startPosX + 72, _startPosY + 68, 6, 6);
                g.FillEllipse(additionalBrush, _startPosX + 92, _startPosY + 68, 6, 6);
            }
        }
    }
}
